class Response:
    """ an unified Response model for every panels
    """

    __dict__ = ('status_code', 'real_resp')

    def __init__(
        self,
        status_code: int,
        real_resp: dict,
     ) -> None:
        """ TODO write doc string """
        self.status_code = status_code
        self.real_resp = real_resp

    def __eq__(self, other) -> bool:
        """checks the equality of two Response object """
        if isinstance(other, Response):
            return self.status_code == other.status_code
        raise TypeError

    def get_real_resp(self):
        return self.real_resp



from prsmsp.models.auth import APIKeyAuth, UnamePassAuth


class AuthFactory:

    @staticmethod
    def get(auth_type):
        auth_types = {
            'api_key': APIKeyAuth,
            'username_pass': UnamePassAuth,
        }

        Auth = auth_types.get(auth_type, None)

        if Auth is None:
            # if the auth method is not supported
            raise ValueError

        # this will not initiate the object, its just returning.
        # if you write like Auth() it will be initiated.
        # this will avoid the Input Error Execption.
        return Auth



import json

import requests

from prsmsp.abctracts.abcpanel import ABCSmsPanel
from prsmsp.factories import AuthFactory
from prsmsp.models import Response


class WebOneSms(ABCSmsPanel):

    def __init__(self, username, password):
        """Take the auth info

        :param username: webonesms username
        :type username: str

        :param password: webonesms password
        :type password: str

        :rtype None
        :return: None
        """
        self.auth = AuthFactory.get('username_pass')(username, password)

    def _response_parser(self, resp):
        status_code = int(resp.status_code)
        try:
            real_response = json.loads(resp.text)
        except json.JSONDecodeError:
            # if the json coder couldn't decode,
            # we just put it like what is is
            real_response = {'resp': resp.text}

        return Response(status_code, real_response)

    def send_sms(self, receptor: str, message: str, originator: str):
        """send sms with webone-sms.ir sms panel

        :param receptor: reciver of your message
        :type receptor: str

        :param message: the message you want to send
        :type message: str

        :param originator: the originator that you want to send your message
        :type originator: str

        :rtype Response
        :return: The requests response
        """

        url = "http://webone-sms.ir/SMSInOutBox/sendsms"
        params = {
            "username": self.auth.username,
            "password": self.auth.password,
            "from": originator,
            "to": receptor,
            "text": message,
        }

        resp = requests.get(url, params=params)

        return self._response_parser(resp)



import json

import requests

from prsmsp.abctracts.abcpanel import ABCSmsPanel
from prsmsp.factories import AuthFactory
from prsmsp.models import Response


class MeliPayamak(ABCSmsPanel):

    def __init__(self, username, password):
        """Take the auth info

        :param username: webonesms username
        :type username: str

        :param password: webonesms password
        :type password: str

        :rtype None
        :return: None
        """
        self.auth = AuthFactory.get('username_pass')(username, password)

    def _response_parser(self, resp):
        status_code = int(resp.status_code)
        real_response = json.loads(resp.text)

        return Response(status_code, real_response)

    def send_sms(self, receptor: str, message: str, originator: str):
        """send sms with melipayamak sms panel

        :param receptor: reciver of your message
        :type receptor: str

        :param message: the message you want to send
        :type message: str

        :param originator: the originator that you want to send your message
        :type originator: str

        :rtype Response
        :return: The requests response
        """

        url = "https://rest.payamak-panel.com/api/SendSMS/SendSMS"

        data = {
                "to": receptor,
                "from": originator,
                "text": message,
                "isFlash": False,
                "username": self.auth.username,
                "password": self.auth.password,
            }

        resp = requests.post(url, json=data)

        return self._response_parser(resp)
